@@ -3,6 +3,7 @@ import shutil
 import torch
 import numpy as np
 import pandas as pd
+from datasets import load_metric
 from tqdm.auto import tqdm
 from transformers import (
     AdamW,
@@ -18,7 +19,6 @@ from pytorch_lightning import Trainer
 from getpass import getpass
 from huggingface_hub import HfApi, Repository
 from pathlib import Path
-from torchmetrics import Accuracy
 
 torch.cuda.empty_cache()
 pl.seed_everything(42)
@@ -30,11 +30,11 @@ class DataModule(Dataset):
     """
 
     def __init__(
-            self,
-            data: pd.DataFrame,
-            tokenizer: T5Tokenizer,
-            source_max_token_len: int = 512,
-            target_max_token_len: int = 512,
+        self,
+        data: pd.DataFrame,
+        tokenizer: T5Tokenizer,
+        source_max_token_len: int = 512,
+        target_max_token_len: int = 512,
     ):
         """
 
@@ -75,9 +75,7 @@ class DataModule(Dataset):
         )
 
         labels = text_encoding["input_ids"]
-        labels[
-            labels == 0
-            ] = -100
+        labels[labels == 0] = -100
 
         return dict(
             keywords=data_row["keywords"],
@@ -91,14 +89,14 @@ class DataModule(Dataset):
 
 class PLDataModule(LightningDataModule):
     def __init__(
-            self,
-            train_df: pd.DataFrame,
-            test_df: pd.DataFrame,
-            tokenizer: T5Tokenizer,
-            source_max_token_len: int = 512,
-            target_max_token_len: int = 512,
-            batch_size: int = 4,
-            split: float = 0.1
+        self,
+        train_df: pd.DataFrame,
+        test_df: pd.DataFrame,
+        tokenizer: T5Tokenizer,
+        source_max_token_len: int = 512,
+        target_max_token_len: int = 512,
+        batch_size: int = 4,
+        split: float = 0.1,
     ):
         """
 
@@ -133,26 +131,26 @@ class PLDataModule(LightningDataModule):
         )
 
     def train_dataloader(self):
-        """ training dataloader """
+        """training dataloader"""
         return DataLoader(
             self.train_dataset, batch_size=self.batch_size, shuffle=True, num_workers=2
         )
 
     def test_dataloader(self):
-        """ test dataloader """
+        """test dataloader"""
         return DataLoader(
             self.test_dataset, batch_size=self.batch_size, shuffle=False, num_workers=2
         )
 
     def val_dataloader(self):
-        """ validation dataloader """
+        """validation dataloader"""
         return DataLoader(
             self.test_dataset, batch_size=self.batch_size, shuffle=False, num_workers=2
         )
 
 
 class LightningModel(pl.LightningModule):
-    """ PyTorch Lightning Model class"""
+    """PyTorch Lightning Model class"""
 
     def __init__(self, tokenizer, model, output: str = "outputs"):
         """
@@ -170,7 +168,7 @@ class LightningModel(pl.LightningModule):
         # self.train_acc = Accuracy()
 
     def forward(self, input_ids, attention_mask, decoder_attention_mask, labels=None):
-        """ forward step """
+        """forward step"""
         output = self.model(
             input_ids,
             attention_mask=attention_mask,
@@ -181,7 +179,7 @@ class LightningModel(pl.LightningModule):
         return output.loss, output.logits
 
     def training_step(self, batch, batch_size):
-        """ training step """
+        """training step"""
         input_ids = batch["keywords_input_ids"]
         attention_mask = batch["keywords_attention_mask"]
         labels = batch["labels"]
@@ -197,7 +195,7 @@ class LightningModel(pl.LightningModule):
         return loss
 
     def validation_step(self, batch, batch_size):
-        """ validation step """
+        """validation step"""
         input_ids = batch["keywords_input_ids"]
         attention_mask = batch["keywords_attention_mask"]
         labels = batch["labels"]
@@ -213,7 +211,7 @@ class LightningModel(pl.LightningModule):
         return loss
 
     def test_step(self, batch, batch_size):
-        """ test step """
+        """test step"""
         input_ids = batch["keywords_input_ids"]
         attention_mask = batch["keywords_attention_mask"]
         labels = batch["labels"]
@@ -230,7 +228,7 @@ class LightningModel(pl.LightningModule):
         return loss
 
     def configure_optimizers(self):
-        """ configure optimizers """
+        """configure optimizers"""
         return AdamW(self.parameters(), lr=0.0001)
 
 
@@ -254,17 +252,17 @@ class trainer:
         )
 
     def train(
-            self,
-            train_df: pd.DataFrame,
-            test_df: pd.DataFrame,
-            source_max_token_len: int = 512,
-            target_max_token_len: int = 512,
-            batch_size: int = 8,
-            max_epochs: int = 5,
-            use_gpu: bool = True,
-            outputdir: str = "outputs",
-            early_stopping_patience_epochs: int = 0,  # 0 to disable early stopping feature
-            test_split=0.1,
+        self,
+        train_df: pd.DataFrame,
+        test_df: pd.DataFrame,
+        source_max_token_len: int = 512,
+        target_max_token_len: int = 512,
+        batch_size: int = 8,
+        max_epochs: int = 5,
+        use_gpu: bool = True,
+        outputdir: str = "outputs",
+        early_stopping_patience_epochs: int = 0,  # 0 to disable early stopping feature
+        test_split=0.1,
     ):
         """
         trains T5 model on custom dataset
@@ -276,7 +274,9 @@ class trainer:
             max_epochs (int, optional): max number of epochs. Defaults to 5.
             use_gpu (bool, optional): if True, model uses gpu for training. Defaults to True.
             outputdir (str, optional): output directory to save model checkpoints. Defaults to "outputs".
-            early_stopping_patience_epochs (int, optional): monitors val_loss on epoch end and stops training, if val_loss does not improve after the specied number of epochs. set 0 to disable early stopping. Defaults to 0 (disabled)
+            early_stopping_patience_epochs (int, optional): monitors val_loss on epoch end and stops training,
+             if val_loss does not improve after the specied number of epochs. set 0 to disable early stopping.
+             Defaults to 0 (disabled)
             :param test_df:
             :param train_df:
         """
@@ -292,7 +292,7 @@ class trainer:
             batch_size=batch_size,
             source_max_token_len=source_max_token_len,
             target_max_token_len=target_max_token_len,
-            split=test_split
+            split=test_split,
         )
 
         self.T5Model = LightningModel(
@@ -327,9 +327,7 @@ class trainer:
 
         trainer.fit(self.T5Model, self.data_module)
 
-    def load_model(
-            self, model_dir: str = "outputs", use_gpu: bool = False
-    ):
+    def load_model(self, model_dir: str = "outputs", use_gpu: bool = False):
         """
         loads a checkpoint for inferencing/prediction
         Args:
@@ -343,16 +341,15 @@ class trainer:
             if torch.cuda.is_available():
                 self.device = torch.device("cuda")
             else:
-                raise Exception("exception ---> no gpu found. set use_gpu=False, to use CPU")
+                raise Exception(
+                    "exception ---> no gpu found. set use_gpu=False, to use CPU"
+                )
         else:
             self.device = torch.device("cpu")
 
         self.model = self.model.to(self.device)
 
-    def save_model(
-            self,
-            model_dir="model"
-    ):
+    def save_model(self, model_dir="model"):
         """
         Save model to dir
         :param model_dir:
@@ -363,20 +360,20 @@ class trainer:
         self.model.save_pretrained(path)
 
     def predict(
-            self,
-            keywords: list,
-            max_length: int = 512,
-            num_return_sequences: int = 1,
-            num_beams: int = 2,
-            top_k: int = 50,
-            top_p: float = 0.95,
-            do_sample: bool = True,
-            repetition_penalty: float = 2.5,
-            length_penalty: float = 1.0,
-            early_stopping: bool = True,
-            skip_special_tokens: bool = True,
-            clean_up_tokenization_spaces: bool = True,
-            use_gpu: bool = True
+        self,
+        keywords: list,
+        max_length: int = 512,
+        num_return_sequences: int = 1,
+        num_beams: int = 2,
+        top_k: int = 50,
+        top_p: float = 0.95,
+        do_sample: bool = True,
+        repetition_penalty: float = 2.5,
+        length_penalty: float = 1.0,
+        early_stopping: bool = True,
+        skip_special_tokens: bool = True,
+        clean_up_tokenization_spaces: bool = True,
+        use_gpu: bool = True,
     ):
         """
         generates prediction for K2T model
@@ -401,11 +398,13 @@ class trainer:
             if torch.cuda.is_available():
                 self.device = torch.device("cuda")
             else:
-                raise Exception("exception ---> no gpu found. set use_gpu=False, to use CPU")
+                raise Exception(
+                    "exception ---> no gpu found. set use_gpu=False, to use CPU"
+                )
         else:
             self.device = torch.device("cpu")
 
-        source_text = ' '.join(map(str, keywords))
+        source_text = " ".join(map(str, keywords))
 
         input_ids = self.tokenizer.encode(
             source_text, return_tensors="pt", add_special_tokens=True
@@ -433,15 +432,82 @@ class trainer:
         ]
         return preds[0]
 
+    def evaluate(self, test_df: pd.DataFrame, metrics: str = "rouge"):
+        """
+
+        :param test_df:
+        :param metrics:
+        :return: Output metrics for keytotext
+        """
+        metric = load_metric(metrics)
+        input_text = test_df["input_text"]
+        references = test_df["output_text"]
+        predictions = [self.predict(x) for x in input_text]
+
+        results = metric.compute(predictions=predictions, references=references)
+
+        output = {
+            "Rouge 1": {
+                "Rouge_1 Low Precision": results["rouge1"].low.precision,
+                "Rouge_1 Low recall": results["rouge1"].low.recall,
+                "Rouge_1 Low F1": results["rouge1"].low.fmeasure,
+                "Rouge_1 Mid Precision": results["rouge1"].mid.precision,
+                "Rouge_1 Mid recall": results["rouge1"].mid.recall,
+                "Rouge_1 Mid F1": results["rouge1"].mid.fmeasure,
+                "Rouge_1 High Precision": results["rouge1"].high.precision,
+                "Rouge_1 High recall": results["rouge1"].high.recall,
+                "Rouge_1 High F1": results["rouge1"].high.fmeasure,
+            },
+            "Rouge 2": {
+                "Rouge_2 Low Precision": results["rouge2"].low.precision,
+                "Rouge_2 Low recall": results["rouge2"].low.recall,
+                "Rouge_2 Low F1": results["rouge2"].low.fmeasure,
+                "Rouge_2 Mid Precision": results["rouge2"].mid.precision,
+                "Rouge_2 Mid recall": results["rouge2"].mid.recall,
+                "Rouge_2 Mid F1": results["rouge2"].mid.fmeasure,
+                "Rouge_2 High Precision": results["rouge2"].high.precision,
+                "Rouge_2 High recall": results["rouge2"].high.recall,
+                "Rouge_2 High F1": results["rouge2"].high.fmeasure,
+            },
+            "Rouge L": {
+                "Rouge_L Low Precision": results["rougeL"].low.precision,
+                "Rouge_L Low recall": results["rougeL"].low.recall,
+                "Rouge_L Low F1": results["rougeL"].low.fmeasure,
+                "Rouge_L Mid Precision": results["rougeL"].mid.precision,
+                "Rouge_L Mid recall": results["rougeL"].mid.recall,
+                "Rouge_L Mid F1": results["rougeL"].mid.fmeasure,
+                "Rouge_L High Precision": results["rougeL"].high.precision,
+                "Rouge_L High recall": results["rougeL"].high.recall,
+                "Rouge_L High F1": results["rougeL"].high.fmeasure,
+            },
+            "rougeLsum": {
+                "rougeLsum Low Precision": results["rougeLsum"].low.precision,
+                "rougeLsum Low recall": results["rougeLsum"].low.recall,
+                "rougeLsum Low F1": results["rougeLsum"].low.fmeasure,
+                "rougeLsum Mid Precision": results["rougeLsum"].mid.precision,
+                "rougeLsum Mid recall": results["rougeLsum"].mid.recall,
+                "rougeLsum Mid F1": results["rougeLsum"].mid.fmeasure,
+                "rougeLsum High Precision": results["rougeLsum"].high.precision,
+                "rougeLsum High recall": results["rougeLsum"].high.recall,
+                "rougeLsum High F1": results["rougeLsum"].high.fmeasure,
+            },
+        }
+        return output
+
     def upload(self, hf_username, model_name):
         hf_password = getpass("Enter your HuggingFace password")
-        if Path('./model').exists():
-            shutil.rmtree('./model')
+        if Path("./model").exists():
+            shutil.rmtree("./model")
         token = HfApi().login(username=hf_username, password=hf_password)
         del hf_password
         model_url = HfApi().create_repo(token=token, name=model_name, exist_ok=True)
-        model_repo = Repository("./model", clone_from=model_url, use_auth_token=token,
-                                git_email=f"{hf_username}@users.noreply.huggingface.co", git_user=hf_username)
+        model_repo = Repository(
+            "./model",
+            clone_from=model_url,
+            use_auth_token=token,
+            git_email=f"{hf_username}@users.noreply.huggingface.co",
+            git_user=hf_username,
+        )
 
         readme_txt = f"""
         ---
@@ -451,33 +517,10 @@ tags:
 - keytotext
 - k2t
 - Keywords to Sentences
-license: "MIT"
-datasets:
-- WebNLG
-- Dart 
-metrics:
-- NLG
 
 model-index:
 - name: {model_name}
 ---
-        
-#keytotext
-
-[![pypi Version](https://img.shields.io/pypi/v/keytotext.svg?logo=pypi&logoColor=white)](https://pypi.org/project/keytotext/)
-[![Downloads](https://static.pepy.tech/personalized-badge/keytotext?period=total&units=none&left_color=grey&right_color=orange&left_text=Pip%20Downloads)](https://pepy.tech/project/keytotext)
-[![Open In Colab](https://colab.research.google.com/assets/colab-badge.svg)](https://colab.research.google.com/github/gagan3012/keytotext/blob/master/notebooks/K2T.ipynb)
-[![Streamlit App](https://static.streamlit.io/badges/streamlit_badge_black_white.svg)](https://share.streamlit.io/gagan3012/keytotext/UI/app.py)
-[![API Call](https://img.shields.io/badge/-FastAPI-red?logo=fastapi&labelColor=white)](https://github.com/gagan3012/keytotext#api)
-[![Docker Call](https://img.shields.io/badge/-Docker%20Image-blue?logo=docker&labelColor=white)](https://hub.docker.com/r/gagan30/keytotext)
-[![HuggingFace](https://img.shields.io/badge/%F0%9F%A4%97-Models%20on%20Hub-yellow)](https://huggingface.co/models?filter=keytotext)
-[![Documentation Status](https://readthedocs.org/projects/keytotext/badge/?version=latest)](https://keytotext.readthedocs.io/en/latest/?badge=latest)
-[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)
-
-
-
-![keytotext](https://socialify.git.ci/gagan3012/keytotext/image?description=1&forks=1&language=1&owner=1&stargazers=1&theme=Light)
-
 
 Idea is to build a model which will take keywords as inputs and generate sentences as outputs.
 
@@ -488,7 +531,7 @@ Potential use case can include:
 - Fine tuning of topic modeling models 
         """.strip()
 
-        (Path(model_repo.local_dir) / 'README.md').write_text(readme_txt)
+        (Path(model_repo.local_dir) / "README.md").write_text(readme_txt)
         self.save_model()
         commit_url = model_repo.push_to_hub()
 
